import { AgentContext } from '@credo-ts/core';
import { AskarStoreExportOptions, AskarStoreImportOptions, AskarStoreRotateKeyOptions } from './AskarApiOptions';
import { AskarModuleConfig } from './AskarModuleConfig';
import { AskarStoreManager } from './AskarStoreManager';
export declare class AskarApi {
    private agentContext;
    private askarStoreManager;
    readonly config: AskarModuleConfig;
    constructor(agentContext: AgentContext, askarStoreManager: AskarStoreManager, config: AskarModuleConfig);
    get isStoreOpen(): boolean;
    /**
     * @throws {AskarStoreDuplicateError} if the wallet already exists
     * @throws {AskarStoreError} if another error occurs
     */
    provisionStore(): Promise<void>;
    /**
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    openStore(): Promise<void>;
    /**
     * Rotate the key of the current askar store.
     *
     * NOTE: multiple agent contexts (tenants) can use the same store. This method rotates the key for the whole store,
     * it is advised to only run this method on the root tenant agent when using profile per wallet database strategy.
     * After running this method you should change the store configuration in the Askar module.
     *
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    rotateStoreKey(options: AskarStoreRotateKeyOptions): Promise<void>;
    /**
     * Exports the current askar store.
     *
     * NOTE: a store can contain profiles for multiple tenants. When you export a store
     * all profiles will be exported with it.
     *
     * NOTE: store must be open before store can be expored
     */
    exportStore(options: AskarStoreExportOptions): Promise<void>;
    /**
     * Imports from an external store config into the current askar store config.
     *
     * NOTE: store must be closed first (using `closeStore`) before store can be imported
     */
    importStore(options: AskarStoreImportOptions): Promise<void>;
    /**
     * Delete the current askar store.
     *
     * NOTE: multiple agent contexts (tenants) can use the same store. This method deletes the whole store.
     *
     *
     * @throws {AskarStoreNotFoundError} if the wallet does not exist
     * @throws {AskarStoreError} if another error occurs
     */
    deleteStore(): Promise<void>;
    /**
     * Close the current askar store.
     *
     * This will close all sessions (also for tenants) in this store.
     */
    closeStore(): Promise<void>;
}
